@@ -6,7 +6,22 @@ use strict;
 use Carp;
 use Regexp::RegGrp;
 
-our $VERSION        = '1.000001';
+our $VERSION            = '1.002';
+
+our @COMPRESS           = ( 'minify', 'pretty' );
+our $DEFAULT_COMPRESS   = 'pretty';
+
+our @BOOLEAN_ACCESSORS = (
+    'no_compress_comment',
+    'remove_copyright'
+);
+
+our @COPYRIGHT_ACCESSORS = (
+    'copyright',
+    'copyright_comment'
+);
+
+our $COPYRIGHT_COMMENT  = '\/\*((?>[^*]|\*[^/])*copyright(?>[^*]|\*[^/])*)\*\/';
 
 our $DICTIONARY     = {
     'STRING1'   => qr~"(?>(?:(?>[^"\\]+)|\\.|\\"|\\\s)*)"~,
@@ -25,18 +40,87 @@ our $MEDIA          = '\@media([^{}]+)\{((?:' . $IMPORT . '|' . $RULE . '|' . $W
 
 our $DECLARATION    = '((?>[^;:]+)):(?<=:)((?>[^;]*))(?:;|\s*$)';
 
-our $COMMENT        = '(\/\*[^*]*\*+([^\/][^*]*\*+)*\/)';
+our $COMMENT        = '(\/\*[^*]*\*+([^/][^*]*\*+)*\/)';
 
 our $PACKER_COMMENT = '\/\*\s*CSS::Packer\s*(\w+)\s*\*\/';
 
 our $CHARSET        = '^(\@charset)\s+(' . $DICTIONARY->{STRING1} . '|' . $DICTIONARY->{STRING2} . ');';
 
+our @REGGRPS        = ( 'whitespaces', 'url', 'import', 'declaration', 'rule', 'content_value', 'mediarules', 'global' );
+
 # --------------------------------------------------------------------------- #
 
+{
+    no strict 'refs';
+
+    foreach my $reggrp ( @REGGRPS ) {
+        next if defined *{ __PACKAGE__ . '::reggrp_' . $reggrp }{CODE};
+
+        *{ __PACKAGE__ . '::reggrp_' . $reggrp } = sub {
+            my ( $self ) = shift;
+
+            return $self->{ '_reggrp_' . $reggrp };
+        };
+    }
+
+    foreach my $field ( @BOOLEAN_ACCESSORS ) {
+        next if defined *{ __PACKAGE__ . '::' . $field }{CODE};
+
+        *{ __PACKAGE__ . '::' . $field} = sub {
+            my ( $self, $value ) = @_;
+
+            $self->{'_' . $field} = $value ? 1 : undef if ( defined( $value ) );
+
+            return $self->{'_' . $field};
+        };
+    }
+
+    foreach my $field ( @COPYRIGHT_ACCESSORS ) {
+        $field = '_' . $field if ( $field eq 'copyright_comment' );
+        next if defined *{ __PACKAGE__ . '::' . $field }{CODE};
+
+        *{ __PACKAGE__ . '::' . $field} = sub {
+            my ( $self, $value ) = @_;
+
+            if ( defined( $value ) and not ref( $value ) ) {
+                $value =~ s/^\s*|\s*$//gs;
+                $self->{'_' . $field} = $value;
+            }
+
+            my $ret = '';
+
+            if ( $self->{'_' . $field} ) {
+                $ret = '/* ' . $self->{'_' . $field} . ' */' . "\n";
+            }
+
+            return $ret;
+        };
+    }
+}
+
+sub compress {
+    my ( $self, $value ) = @_;
+
+    if ( defined( $value ) ) {
+        if ( grep( $value eq $_, @COMPRESS ) ) {
+            $self->{_compress} = $value;
+        }
+        elsif ( ! $value ) {
+            $self->{_compress} = undef;
+        }
+    }
+
+    $self->{_compress} ||= $DEFAULT_COMPRESS;
+
+    return $self->{_compress};
+}
+
 sub init {
-    my $class    = shift;
+    my $class   = shift;
     my $self    = {};
 
+    bless( $self, $class );
+
     $self->{content_value}->{reggrp_data} = [
         {
             regexp      => $DICTIONARY->{STRING1}
@@ -81,15 +165,10 @@ sub init {
                 my $submatches  = $_[0]->{submatches};
                 my $url         = $submatches->[0];
                 my $mediatype   = $submatches->[2];
-                my $opts        = $_[0]->{opts} || {};
 
-                my $compress    = _get_opt( $opts, 'compress' );
+                my $compress    = $self->compress();
 
-                # I don't like this, but
-                # $self->{url}->{reggrp}->exec( \$url );
-                # will not work. It isn't initialized jet.
-                # If someone has a better idea, please let me know
-                $self->_process_wrapper( 'url', \$url, $opts );
+                $self->reggrp_url()->exec( \$url );
 
                 $mediatype =~ s/^\s*|\s*$//gs;
                 $mediatype =~ s/\s*,\s*/,/gsm;
@@ -106,19 +185,14 @@ sub init {
                 my $submatches  = $_[0]->{submatches};
                 my $key         = $submatches->[0];
                 my $value       = $submatches->[1];
-                my $opts        = $_[0]->{opts} || {};
 
-                my $compress    = _get_opt( $opts, 'compress' );
+                my $compress    = $self->compress();
 
                 $key    =~ s/^\s*|\s*$//gs;
                 $value  =~ s/^\s*|\s*$//gs;
 
                 if ( $key eq 'content' ) {
-                    # I don't like this, but
-                    # $self->{content_value}->{reggrp}->exec( \$value );
-                    # will not work. It isn't initialized jet.
-                    # If someone has a better idea, please let me know
-                    $self->_process_wrapper( 'content_value', \$value, $opts );
+                    $self->reggrp_content_value->exec( \$value );
                 }
                 else {
                     $value =~ s/\s*,\s*/,/gsm;
@@ -139,9 +213,8 @@ sub init {
                 my $submatches  = $_[0]->{submatches};
                 my $selector    = $submatches->[0];
                 my $declaration = $submatches->[1];
-                my $opts        = $_[0]->{opts} || {};
 
-                my $compress    = _get_opt( $opts, 'compress' );
+                my $compress    = $self->compress();
 
                 $selector =~ s/^\s*|\s*$//gs;
                 $selector =~ s/\s*,\s*/,/gsm;
@@ -149,11 +222,7 @@ sub init {
 
                 $declaration =~ s/^\s*|\s*$//gs;
 
-                # I don't like this, but
-                # $self->{declaration}->{reggrp}->exec( \$declaration );
-                # will not work. It isn't initialized jet.
-                # If someone has a better idea, please let me know
-                $self->_process_wrapper( 'declaration', \$declaration, $opts );
+                $self->reggrp_declaration()->exec( \$declaration );
 
                 my $store = $selector . '{' . ( $compress eq 'pretty' ? "\n" : '' ) . $declaration . '}' .
                     ( $compress eq 'pretty' ? "\n" : '' );
@@ -176,9 +245,10 @@ sub init {
             regexp      => $CHARSET,
             replacement => sub {
                 my $submatches  = $_[0]->{submatches};
-                my $opts        = $_[0]->{opts} || {};
 
-                return $submatches->[0] . " " . $submatches->[1] . ( $opts->{compress} eq 'pretty' ? "\n" : '' );
+                my $compress    = $self->compress();
+
+                return $submatches->[0] . " " . $submatches->[1] . ( $compress eq 'pretty' ? "\n" : '' );
             }
         },
         {
@@ -187,18 +257,13 @@ sub init {
                 my $submatches  = $_[0]->{submatches};
                 my $mediatype   = $submatches->[0];
                 my $mediarules  = $submatches->[1];
-                my $opts        = $_[0]->{opts} || {};
 
-                my $compress    = _get_opt( $opts, 'compress' );
+                my $compress    = $self->compress();
 
                 $mediatype =~ s/^\s*|\s*$//gs;
                 $mediatype =~ s/\s*,\s*/,/gsm;
 
-                # I don't like this, but
-                # $self->{mediarules}->{reggrp}->exec( \$mediarules );
-                # will not work. It isn't initialized jet.
-                # If someone has a better idea, please let me know
-                $self->_process_wrapper( 'mediarules', \$mediarules, $opts );
+                $self->reggrp_mediarules()->exec( \$mediarules );
 
                 return '@media ' . $mediatype . '{' . ( $compress eq 'pretty' ? "\n" : '' ) .
                     $mediarules . '}' . ( $compress eq 'pretty' ? "\n" : '' );
@@ -209,14 +274,12 @@ sub init {
 
 
     map {
-        $self->{$_}->{reggrp} = Regexp::RegGrp->new(
+        $self->{ '_reggrp_' . $_ } = Regexp::RegGrp->new(
             {
                 reggrp => $self->{$_}->{reggrp_data}
             }
         );
-    } ( 'whitespaces', 'url', 'import', 'declaration', 'rule', 'content_value', 'mediarules', 'global' );
-
-    bless( $self, $class );
+    } @REGGRPS;
 
     return $self;
 }
@@ -256,56 +319,44 @@ sub minify {
         $css = ref( $input ) ? $input : \$input;
     }
 
-    if ( ref( $opts ) ne 'HASH' ) {
-        carp( 'Second argument must be a hashref of options! Using defaults!' ) if ( $opts );
-        $opts = { compress => 'pretty', no_compress_comment => 0 };
-    }
-    else {
-        $opts->{compress}               = grep( $opts->{compress}, ( 'minify', 'pretty' ) ) ? $opts->{compress} : 'pretty';
-        $opts->{no_compress_comment}    = $opts->{no_compress_comment} ? 1 : 0;
+    if ( ref( $opts ) eq 'HASH' ) {
+        foreach my $field ( @BOOLEAN_ACCESSORS ) {
+            $self->$field( $opts->{$field} ) if ( defined( $opts->{$field} ) );
+        }
+
+        foreach my $field ( 'compress', 'copyright' ) {
+            $self->$field( $opts->{$field} ) if ( defined( $opts->{$field} ) );
+        }
     }
 
-    if ( not $opts->{no_compress_comment} and ${$css} =~ /$PACKER_COMMENT/ ) {
+    my $copyright_comment = '';
+
+    if ( ${$css} =~ /$COPYRIGHT_COMMENT/ism ) {
+        $copyright_comment = $1;
+    }
+    # Resets copyright_comment() if there is no copyright comment
+    $self->_copyright_comment( $copyright_comment );
+
+    if ( not $self->no_compress_comment() and ${$css} =~ /$PACKER_COMMENT/ ) {
         my $compress = $1;
         if ( $compress eq '_no_compress_' ) {
             return ( $cont eq 'scalar' ) ? ${$css} : undef;
         }
 
-        $opts->{compress} = grep( $compress, ( 'minify', 'pretty' ) ) ? $compress : $opts->{compress};
+        $self->compress( $compress );
     }
 
     ${$css} =~ s/$COMMENT/ /gsm;
 
-    $self->{global}->{reggrp}->exec( $css, $opts );
+    $self->reggrp_global()->exec( $css );
+
+    if ( not $self->remove_copyright() ) {
+        ${$css} = ( $self->copyright() || $self->_copyright_comment() ) . ${$css};
+    }
 
     return ${$css} if ( $cont eq 'scalar' );
 }
 
-sub _process_wrapper {
-    my ( $self, $reg_name, $in, $opts ) = @_;
-
-    $self->{$reg_name}->{reggrp}->exec( $in, $opts );
-}
-
-sub _restore_wrapper {
-    my ( $self, $reg_name, $in ) = @_;
-
-    $self->{$reg_name}->{reggrp}->restore_stored( $in );
-}
-
-sub _get_opt {
-    my ( $opts_hash, $opt ) = @_;
-
-    $opts_hash  ||= {};
-    $opt       ||= '';
-
-    my $ret = '';
-
-    $ret = $opts_hash->{$opt} if ( defined( $opts_hash->{$opt} ) );
-
-    return $ret;
-}
-
 1;
 
 __END__
@@ -316,7 +367,7 @@ CSS::Packer - Another CSS minifier
 
 =head1 VERSION
 
-Version 1.000001
+Version 1.002
 
 =head1 DESCRIPTION
 
@@ -339,7 +390,7 @@ For backward compatibility it is still possible to call 'minify' as a function:
     CSS::Packer::minify( $scalarref, $opts );
 
 First argument must be a scalarref of CSS-Code.
-Second argument must be a hashref of options. The only option is
+Second argument must be a hashref of options. Possible options are:
 
 =over 4
 
@@ -370,6 +421,26 @@ to
 
     a{color:black;}div{width:100px;}
 
+=item copyright
+
+You can add a copyright notice at the top of the script.
+
+=item remove_copyright
+
+If there is a copyright notice in a comment it will only be removed if this
+option is set to a true value. Otherwise the first comment that contains the
+word "copyright" will be added at the top of the packed script. A copyright
+comment will be overwritten by a copyright notice defined with the copyright
+option.
+
+=item no_compress_comment
+
+If not set to a true value it is allowed to set a CSS comment that
+prevents the input being packed or defines a compression level.
+
+    /* CSS::Packer _no_compress_ */
+    /* CSS::Packer pretty */
+
 =back
 
 =head1 AUTHOR
@@ -378,8 +449,8 @@ Merten Falk, C<< <nevesenin at cpan.org> >>
 
 =head1 BUGS
 
-Please report any bugs or feature requests to C<bug-css-packer at rt.cpan.org>, or through
-the web interface at L<http://rt.cpan.org/NoAuth/ReportBug.html?Queue=CSS-Packer>.  I will be notified, and then you'll
+Please report any bugs or feature requests through
+the web interface at L<http://github.com/nevesenin/css-packer-perl/issues>.  I will be notified, and then you'll
 automatically be notified of progress on your bug as I make changes.
 
 =head1 SUPPORT
